from math import isnan
from typing import *

import pytest
from hypothesis import given, infer, assume
from hypothesis.strategies import lists, floats

from examples.map_job import (
    ones_and_zeros,
    divide_by,
    times_3,
    sum_up,
    filter_division_errors,
    Error,
)


def test_ones_and_zeros(generic_event, generic_context):
    result = ones_and_zeros(generic_event, generic_context)
    assert len(result) > 5 and isinstance(result, list)
    assert all(n in [0, 1] for n in result)


def test_divide_by(generic_event, generic_context):
    numbers = ones_and_zeros(generic_event, generic_context)
    for n in numbers:
        if n == 0:
            with pytest.raises(ZeroDivisionError):
                divide_by(n, generic_context)
        else:
            assert divide_by(n, generic_context)


@given(n=infer)
def test_times_3(n: Union[int, float], generic_context):
    assume(not isnan(n))
    assert times_3(n, generic_context) == n * 3


@given(numbers=lists(floats(min_value=0)))
def test_sum_up(numbers: List[float], generic_context):
    assert sum_up(numbers, generic_context) == sum(numbers)


@given(event=infer)
def test_division_error_filter(event: Union[float, Error], generic_context):
    result = filter_division_errors(event, generic_context)
    if isinstance(event, float):
        assume(not isnan(event))
        assert result == event
    else:
        assert result == 0.0



from dataclasses import dataclass

from orkestra.interfaces import Nextable


@dataclass
class GenericLambdaContext:
    function_name: str = "test"
    memory_limit_in_mb: int = 128
    invoked_function_arn: str = (
        "arn:aws:lambda:eu-west-1:809313241:function:test"
    )
    aws_request_id: str = "52fdfc07-2182-154f-163f-5f0f9a621d72"


def coerce(obj: Nextable) -> Nextable:
    """
    Overload the `__rshift__` operator of obj to call its .next() method and do the same for the object it's called on.

    Args:
        obj: an object with a `.next(self, other)` method

    Returns: obj

    """

    def rshift(self, right):
        result = self.next(right)
        coerce(right)
        coerce(result)
        return result

    obj.__class__.__rshift__ = rshift
    return obj


def _cdk_patch(kwargs: dict):
    """
    Replace interface elements in kwargs with their cdk counterpart.
    """

    for key, value in kwargs.items():

        if hasattr(value, "cdk_construct"):

            kwargs[key] = value.cdk_construct

    return kwargs


def _coalesce(*dictionaries: dict, **kwargs):
    """
    Coalesce multiple dictionaries into one, passing only non-None arguments.
    """

    def filter_none(dict):
        return {k: v for k, v in dict.items() if v is not None}

    result, *dictionaries = dictionaries

    result = result.copy()

    for d in dictionaries:

        result.update(filter_none(d))

    result.update(filter_none(kwargs))

    _cdk_patch(result)

    return result


generic_context = GenericLambdaContext()



import logging

logger = logging.getLogger(__name__)

logger.addHandler(logging.NullHandler())


from orkestra.decorators import compose, powertools
from orkestra.utils import coerce, generic_context



import pytest

from examples.hello_orkestra import (
    generate_item,
    add_price,
    copy_item,
    double_price,
    Item,
    assert_false,
    do_nothing,
    say_hello,
    say_goodbye,
    random_int,
    random_float,
)


class TestMethods:
    @staticmethod
    def test_generate_item(generic_event, generic_context):
        generated = generate_item(generic_event, generic_context)
        assert Item(**generated)

    @staticmethod
    def test_add_price(generic_event, generic_context):
        result = add_price(generic_event, generic_context)
        assert result["price"]

    @staticmethod
    def test_copy_item(generic_event, generic_context):
        result = copy_item(generic_event, generic_context)
        assert all(i == generic_event for i in result)

    @staticmethod
    def test_double_price(generic_event, generic_context):
        generic_event["price"] = 1
        result = double_price(generic_event, generic_context)
        assert result["price"] == generic_event["price"] * 2

    @staticmethod
    def test_assert_false(generic_event, generic_context):
        with pytest.raises(AssertionError):
            assert_false(generic_event, generic_context)

    @staticmethod
    def test_do_nothing(generic_event, generic_context):
        assert do_nothing(generic_event, generic_context) is None

    @staticmethod
    def test_say_hello(generic_event, generic_context):
        assert say_hello(generic_event, generic_context)

    @staticmethod
    def test_goodbye(generic_event, generic_context):
        assert say_goodbye(generic_event, generic_context)

    @staticmethod
    def test_random_int(generic_event, generic_context):
        result = random_int(generic_event, generic_context)
        assert isinstance(result, int)

    @staticmethod
    def test_random_float(generic_event, generic_context):
        result = random_float(generic_event, generic_context)
        assert isinstance(result, float)



import pytest

from orkestra import compose
from orkestra.interfaces import AdjacencyList, Composable


@pytest.fixture
def composed():
    @compose
    def noop(event, context):
        ...

    return noop


def test_composable(composed):
    assert isinstance(composed, Composable)


def test_adjacency_list(composed):
    assert isinstance(composed, AdjacencyList)



import pytest

from examples.rest import input_order, process_order


@pytest.fixture
def order():
    return {"id": "foo", "item": "hat"}


def test_input_order(order, generic_context):
    result = input_order(order, generic_context)
    assert result["id"] == order["id"]


def test_composition():
    assert process_order in input_order.downstream


@pytest.mark.slow
def test_process_order(order, generic_context):
    assert process_order(order, generic_context) == order



import os

import boto3


def test_successful_sfn_execution():
    client = boto3.client("stepfunctions")

    response = client.start_sync_execution(
        stateMachineArn=os.environ["SINGLE_LAMBDA_STATE_MACHINE_ARN"],
    )

    assert response["status"] == "SUCCEEDED"
